/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var LaunchyContext = function() {}

LaunchyContext.prototype.doInit = function() {
	// did Launchy already load?
	var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
	var w = ass.hiddenDOMWindow;
	if (typeof(w.launchy) == "undefined") {
		LaunchyContext.version = "2.4.0";
		w.launchy = LaunchyContext;
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefService.getBranch(null);
		try {
			LaunchyContext.dev = pref.getBoolPref("extensions.launchy.developer");
			launchyShared.trace("Developer information is on");
			launchyShared.trace("Launchy version <" + LaunchyContext.version + "> installed");
		} catch(ex) {
		}

		LaunchyContext.launchy = true;
		LaunchyContext.jslib = true;
		try {
			include(jslib_file);
			include(jslib_fileutils);
			include(jslib_dir);
			include(jslib_dirutils);
		} catch(ex) {
			launchyShared.trace("JSLib isn't installed! Launchy will be disabled!");
			LaunchyContext.launchy = false;
			LaunchyContext.jslib = false;
		}

		if (typeof(LaunchyContext.jslib) != "undefined" && LaunchyContext.jslib) {
			launchyShared.trace("JSLib version <" + JS_LIB_VERSION + "> installed");

			LaunchyContext.fu=new FileUtils();
			LaunchyContext.du=new DirUtils();
			LaunchyContext.tmpdir = new Dir(LaunchyContext.du.getTmpDir());

			// cleanup view-source temp files
			launchyContext.doSourceCleanup();

			LaunchyContext.progdir = LaunchyContext.du.getCurProcDir().toLowerCase() + "\\";
			launchyShared.trace("Program directory is <" + LaunchyContext.progdir + ">");

			if ("@mozilla.org/winhooks;1" in Components.classes) {
				try {
					LaunchyContext.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
				} catch(ex) {
					launchyShared.trace("Unable to get winhooks service!");
				}
			} else if ("@mozilla.org/browser/shell-service;1" in Components.classes) {
				try {
					LaunchyContext.winhooks = Components.classes["@mozilla.org/browser/shell-service;1"].getService(Components.interfaces.nsIWindowsShellService);
				} catch(ex) {
					launchyShared.trace("Unable to get browser shell service!");
				}
			} else {
				launchyShared.trace("Unable to get Components. This only works on Windows!");
			}
			if (typeof(LaunchyContext.winhooks) == "undefined" || typeof(LaunchyContext.winhooks.getRegistryEntry) != "function") {
				LaunchyContext.winhooks = false;
				launchyShared.trace("getRegistryEntry isn't available. This only works on Windows!");
			}

			LaunchyContext.apps = new Array();

			if (LaunchyContext.winhooks) {
				launchyShared.trace("Loading applications from JavaScript");

				// Browsers
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla Firebird", "CurrentVersion", "Mozilla Firebird", "", 1, "", 1);
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla Firefox", "CurrentVersion", "Mozilla Firefox", "", 1, "", 1);
				launchyContext.addApp(2, "Software\\mozilla.org\\Netscape", "CurrentVersion", "Netscape", "", 1, "", 1);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Opera.exe", "", "Opera", "", 1, "", 0);
				launchyContext.addApp(2, "Software\\K-Meleon\\K-Meleon\\General", "InstallDir", "K-Meleon", "k-meleon.exe", 1, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\MYIE2", "DisplayIcon", "MyIE2", "", 1, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE", "", "Internet Explorer", "", 1, "", 0);
				launchyContext.addApp(2, "Software\\Siber Systems\\RoboForm", "_TBP_", "Avant Browser", "", 1, "", 0);
				launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Crazy Browser_is1", "DisplayIcon", "Crazy Browser", "", 1, "", 0);
				launchyContext.addApp(3, "Software\\FlashPeak\\SlimBrowser", "", "Slim Browser", "sbrowser.exe", 1, "", 0);

				// Mail clients
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Mail", "", 2, ["-compose"], 1);
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
				launchyContext.addApp(2, "Software\\mozilla.org\\Netscape", "CurrentVersion", "Netscape Mail", "", 2, ["-compose"], 1);
				launchyContext.addApp(2, "Software\\Pegasus Mail\\BaseDir", "", "Pegasus Mail", "wsendto.exe", 2, "", 0);
				launchyContext.addApp(3, "Software\\Lotus\\Notes", "Path", "Lotus Notes", "notes.exe", 2, "", 0);
				launchyContext.addApp(2, "Software\\Qualcomm\\Eudora\\CommandLine", "current", "Eudora", "", 2, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\OUTLOOK.EXE", "", "Outlook", "", 2, ["-c", "IPM.Note", "/m"], 0);
				launchyContext.addApp(2, "Software\\RIT\\The Bat!", "EXE path", "The Bat!", "", 2, "", 0);
				launchyContext.addApp(3, "Software\\IncrediMail", "InstallationPath", "IncrediMail", "bin\\IncMail.exe", 2, ["/c", "/n", "/mailto:#1"] , 0);
				launchyContext.addApp(3, "Software\\MCS\\Calypso3", "Path", "Calypso", "Calypso.exe", 2, ["/MAILTO"] , 0);

				// Special case applications
				var tmp = launchyContext.findApp(3, "Software\\Microsoft\\WAB\\Export\\CSV", "DLL", "Outlook Express");
				if (tmp) {
					var re=/wabimp\.dll/i;
					tmp = tmp.replace(re, "msimn.exe");
					LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp("Outlook Express", tmp, 2, ["/mailurl:#1"]);
				}

				// Media clients
				launchyContext.addApp(2, "Software\\BST\\bsplayerv1", "AppPath", "BSPlayer", "", 3, "", 0);
				launchyContext.addApp(3, "Software\\Clients\\Media\\iTunes\\shell\\open\\command", "", "iTunes", "", 3, ["/url"], 0);
				launchyContext.addApp(2, "Software\\Winamp", "", "Winamp", "winamp.exe", 3, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\MediaPlayer", "Installation Directory", "Windows Media Player", "wmplayer.exe", 3, "", 0);

				// FTP clients
				launchyContext.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 4, ["/O"], 0);
				launchyContext.addApp(2, "Software\\UltraFXP", "", "UltraFXP", "ultrafxp.exe", 4, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\wsftppro.exe", "", "WS_FTP Pro", "", 4, "", 0);
				launchyContext.addApp(3, "Software\\SmartFTP", "Install Directory", "SmartFTP", "smartftp.exe", 4, "", 0);
				launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\BulletProof FTP_is1", "DisplayIcon", "BulletProof FTP", "", 4, "", 0);
				launchyContext.addApp(2, "Software\\FileZilla", "Install_Dir", "FileZilla", "FileZilla.exe", 4, "", 0);
				launchyContext.addApp(2, "Software\\VanDyke\\AbsoluteFTP\\Install", "Main Directory", "AbsoluteFTP", "AbsoluteFTP.EXE", 4, "", 0);

				// Download Managers
				launchyContext.addApp(2, "Software\\HeadLight\\GetRight\\Config", "InstallDir", "GetRight", "getright.exe", 5, "", 0);
				launchyContext.addApp(2, "Software\\Cronosoft\\LeechGet\\Console", "LeechGetPath", "LeechGet", "", 5, ["-wizard"], 0);
				launchyContext.addApp(0, "TypeLib\\{3F9A5301-9E5A-11D3-A288-0000E80E2EDE}\\1.0\\HELPDIR", "", "Mass Downloader", "massdown.exe", 5, "", 0);
				launchyContext.addApp(2, "Software\\Star Downloader", "Path", "Star Downloader", "", 5, "", 0);
				launchyContext.addApp(2, "Software\\DownloadManager", "ExePath", "Internet Download Manager", "", 5, ["/d"], 0);
				launchyContext.addApp(0, "TypeLib\\{1537E849-0200-11D2-8059-000000000000}\\1.0\\HELPDIR", "", "ReGet Deluxe", "regetdx.exe", 5, "", 0);
				launchyContext.addApp(0, "bittorrent\\shell\\open\\command", "", "BitTorrent", "", 5, ["--url"], 0);
				launchyContext.addApp(2, "Software\\JetCar\\JetCar\\General", "AppPath", "FlashGet", "", 5, "", 0);
				launchyContext.addApp(2, "Software\\Radiate\\Go!Zilla", "", "Go!Zilla", "", 5, "", 0);
				launchyContext.addApp(3, "Software\\SpeedBit\\Download Accelerator", "EXELOCATION", "Download Accelerator", "", 5, "", 0);
				launchyContext.addApp(3, "Software\\Millweed\\WackGet", "", "WackGet", "WackGet.exe", 5, "", 0);
				launchyContext.addApp(2, "Software\\MetaProducts\\Offline Explorer Pro\\Parameters", "LastRunPos", "Offline Explorer Pro", "", 5, ["/new=url=#1"], 0);

				// File Explorers
				launchyContext.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 6, ["/O"], 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows NT\\CurrentVersion", "SystemRoot", "Explorer", "explorer.exe", 6, "", 0);
				launchyContext.addApp(2, "Software\\Altap\\Applications\\Servant Salamander 2.5 beta 4", "LastDirectory", "Servant Salamander", "salamand.exe", 6, "", 0);

				// Editors
				launchyContext.addApp(0, "Applications\\notepad.exe\\shell\\edit\\command", "", "Notepad", "", 7, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\UEDIT32.exe", "", "UltraEdit", "", 7, "", 0);
				launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\soffice.exe", "", "OpenOffice.org", "", 7, ["-o"], 0);
				launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Composer", "", 7, ["-editor"], 1);
			}
		}

//		if (typeof(LaunchyContext.apps) != "undefined" && LaunchyContext.apps.length) {
//			for (var j = 0; j < LaunchyContext.apps.length; j++) {
//				launchyShared.trace("name: <" + LaunchyContext.apps[j].name + "> type: <" + LaunchyContext.apps[j].type + "> exe: <" + LaunchyContext.apps[j].exe + "> args: <" + LaunchyContext.apps[j].args + ">");
//			}
//		}

		w.launchy = LaunchyContext;
	} else {
		LaunchyContext = w.launchy;
	}
}

LaunchyContext.prototype.findApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_gecko) {

	// reg values:
	// 0: HKEY_CLASSES_ROOT
	// 1: HKEY_CURRENT_CONFIG
	// 2: HKEY_CURRENT_USER
	// 3: HKEY_LOCAL_MACHINE
	// 4: HKEY_USERS

	var reg_type = new Array("HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG", "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS");
	launchyShared.trace("Looking for <" + app_name + "> in <" + reg_type[reg] + "> at <" + reg_sub + (reg_name?"\\"+reg_name:"") + ">" + (parseInt(app_gecko)?" and it's a Gecko":""));
	try {
		var tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub, reg_name);
		var tmp_gecko;
		if (app_gecko && tmp) {
			launchyShared.trace("Found Gecko version <" + tmp + ">");
			tmp_gecko = tmp;
			tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub + "\\" + tmp + "\\Main", "PathToExe");
		}
	} catch(ex) {
		launchyShared.trace("Unable to find " + app_name + " due to " + ex);
		return false;
	}
	if (tmp) {
		var re=/\\\\/;
		tmp = tmp.replace(re, "\\");
		re=/\\$/;
		tmp = tmp.replace(re, "");
		re=/%1$/;
		tmp = tmp.replace(re, "");
		re=/\s+$/;
		tmp = tmp.replace(re, "");

		launchyShared.trace("<" + app_name + "> registry entry found <" + tmp + ">");
		if (app_exe) {
			tmp += "\\" + app_exe;
		}

		re=/\"(.*?)\" ?/;
		var x = tmp.match(re);
		if (x && x[1]) {
			tmp = x[1];
		}

		if (LaunchyContext.fu.exists(tmp)) {
			var f = new File(tmp);
			if (f.isFile() && f.isExec()) {
				launchyShared.trace(app_name + (app_gecko ? " <" + tmp_gecko + ">" : "") + " is installed at <" + tmp + ">");
				if (app_gecko && tmp.toLowerCase().indexOf(LaunchyContext.progdir) != -1) {
					launchyShared.trace("Hide " + app_name + " since it's the running client");
					return false;
				}
			} else {
				launchyShared.trace("File <" + tmp + "> isn't a file or isn't executable");
				return false;
			}
			return tmp;
		} else {
			launchyShared.trace(app_name + " isn't at: <" + tmp + ">");
			return false;
		}
	} else {
		launchyShared.trace(app_name + " isn't installed or undetected");
		return false;
	}
}

LaunchyContext.prototype.createApp = function(app_name, app_exe, app_type, app_args) {
	var app = new Array();
	app.name = app_name;
	app.exe = app_exe;
	app.type = app_type;
	app.args = app_args;
	return app;
}

LaunchyContext.prototype.addApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_type, app_args, app_gecko) {
	// Check for dupes
	for (var i = 0; LaunchyContext.apps.length > i; i++) {
		if (LaunchyContext.apps[i].name == app_name) {
			if (LaunchyContext.apps[i].type != app_type) {
				launchyShared.trace("Reusing <" + app_name + "> detection");
				LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, LaunchyContext.apps[i].exe, app_type, app_args);
				return;
			} else {
				launchyShared.trace("<" + app_name + "> type <" + app_type + "> is already detected");
				return;
			}
		}
	}
	var tmp = launchyContext.findApp(reg, reg_sub, reg_name, app_name, app_exe, app_gecko);
	if (tmp) {
		LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, tmp, app_type, app_args);
	}
}

LaunchyContext.prototype.doContext = function(i) {
	var menuid = document.getElementById(i);
	var children = menuid.childNodes;
	for (var j = children.length-1; j >= 0; j--) {
		menuid.removeChild(children[j]);
	}

	var label;
	var tooltip;
	var flabel;
	var ftooltip;
	var menuitem;

	if (!LaunchyContext.jslib) {
		label = document.getElementById("bundle_launchy").getString("launchy-disable-nojslib-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-disable-nojslib-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"nojslib\");");
		menuid.appendChild(menuitem);
		return;
	}
	if (!LaunchyContext.winhooks && typeof(LaunchyContext.apps) == "undefined") {
		label = document.getElementById("bundle_launchy").getString("launchy-disable-nowinhooks-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-disable-nowinhooks-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"nowinhooks\");");
		menuid.appendChild(menuitem);
		return;
	}
	if (typeof(LaunchyContext.apps) == "undefined" || !LaunchyContext.apps.length) {
		label = document.getElementById("bundle_launchy").getString("launchy-disable-noapps-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-disable-noapps-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"noapps\");");
		menuid.appendChild(menuitem);
		return;
	}

	var num = 0;

	var sepfound = false;
	// right click on page
	if (!gContextMenu.onLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-page-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-page-tooltip");
		if (gContextMenu.inFrame) {
			flabel = document.getElementById("bundle_launchy").getString("launchy-page-frame-label");
			ftooltip = document.getElementById("bundle_launchy").getString("launchy-page-frame-tooltip");
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\");");
				menuid.appendChild(menuitem);
				num++;
				if (gContextMenu.inFrame) {
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(flabel, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(ftooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 0, 1);");
					menuid.appendChild(menuitem);
				}
			}
		}
		label = document.getElementById("bundle_launchy").getString("launchy-source-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-source-tooltip");
		if (gContextMenu.inFrame) {
			flabel = document.getElementById("bundle_launchy").getString("launchy-source-frame-label");
			ftooltip = document.getElementById("bundle_launchy").getString("launchy-source-frame-tooltip");
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 7) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doSource(\"" + i + "\", 0, 0);");
				menuid.appendChild(menuitem);
				num++;
				if (gContextMenu.inFrame) {
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(flabel, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(ftooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doSource(\"" + i + "\", 0, 1);");
					menuid.appendChild(menuitem);
				}
			}
		}
	}

	// right click on link
	else if (gContextMenu.onLink && gContextMenu.onSaveableLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-link-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-link-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				num++;
			}
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 3) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				num++;
			}
		}
		sepfound = false;
		var url = "" + gContextMenu.link;
		launchyShared.trace("Clicked on url <" + url + ">");
		var re=/^(http:\/\/ftp|ftp:\/\/)/i;
		if (url.match(re)) {
			url = url.replace(/^http:\/\//i, "ftp:\/\/");
			LaunchyContext.url = url;
			label = document.getElementById("bundle_launchy").getString("launchy-ftp-label");
			tooltip = document.getElementById("bundle_launchy").getString("launchy-ftp-tooltip");
			sepfound = false;
			for (i = 0; LaunchyContext.apps.length > i; i++) {
				if (LaunchyContext.apps[i].type == 4) {
					if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
						menuitem = document.createElement("menuseparator");
						menuid.appendChild(menuitem);
						sepfound = true;
					}
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1, 0);");
					menuid.appendChild(menuitem);
					num++;
				}
			}
		}
		label = document.getElementById("bundle_launchy").getString("launchy-download-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-download-tooltip");
		sepfound = false;
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 5) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				num++;
			}
		}
		re=/^file:\/\//i;
		if (url.match(re)) {
			if (/file:\/\/\//.test(url)) {
			} else {
				url = url.replace(/(\w+:\/\/)/, "\\\\");
				launchyShared.trace("Mangled URL <" + url + ">");
				url = url.replace(/(\/)/g, "\\");
				launchyShared.trace("Mangled URL <" + url + ">");
				url = url.replace(/(\+)/g, " ");
				launchyShared.trace("Mangled URL <" + url + ">");
			}
			url = unescape(url);
			launchyShared.trace("Mangled URL <" + url + ">");
			LaunchyContext.url = url;

			label = document.getElementById("bundle_launchy").getString("launchy-file-label");
			tooltip = document.getElementById("bundle_launchy").getString("launchy-file-tooltip");
			sepfound = false;
			for (i = 0; LaunchyContext.apps.length > i; i++) {
				if (LaunchyContext.apps[i].type == 6) {
					if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
						menuitem = document.createElement("menuseparator");
						menuid.appendChild(menuitem);
						sepfound = true;
					}
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
					menuid.appendChild(menuitem);
					num++;
				}
			}
		}
	}

	// right click on mailto link
	else if (gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-mail-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-mail-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 2) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				num++;
			}
		}
	}
	if (!num) {
		label = document.getElementById("bundle_launchy").getString("launchy-disable-noaction-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-disable-noaction-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"noaction\");");
		menuid.appendChild(menuitem);
		return;
	}
}

LaunchyContext.prototype.doApp = function(app_entry, on_link, on_frame) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var url;
	if (typeof(LaunchyContext.url) != "undefined" && LaunchyContext.url) {
		url = LaunchyContext.url;
		LaunchyContext.url = "";
	} else {
		if (on_link) {
			url = gContextMenu.link;
		} else {
			if (on_frame) {
				url = window._content.location.href;
			} else {
				url = document.commandDispatcher.focusedWindow.document.location.href;
			}
		}
	}
	if (!url) {
		launchyShared.trace("Didn't get an URL");
		return;
	}

	var xargs = new Array();
	if (LaunchyContext.apps[app_entry].args)
		xargs = LaunchyContext.apps[app_entry].args.join(',').split(',');

	var u = false;
	for (var i=0; i < xargs.length; i++) {
		var tmp = launchyShared.replaceInsert(xargs[i], 1, url)
		if (tmp != xargs[i]) {
			xargs[i] = tmp;
			u = true;
		}
	}
	if (!u)
		xargs[xargs.length] = url;

	launchyShared.trace("About to run:\n" + app_exe + " " + xargs.join(" "));
	try {
		var stat = document.getElementById("statusbar-display");
		stat.label = "Running <" + app_exe + " " + xargs.join(" ") + ">";
	} catch(err) {
	}
	LaunchyContext.fu.run(app_exe, xargs);
}

LaunchyContext.prototype.doSource = function(app_entry, on_link, on_frame) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var doc;
	if (on_frame) {
		doc = window._content.document;
	} else if (on_link) {
		doc = gContextMenu.link;
	} else {
		doc = document.commandDispatcher.focusedWindow.document;
	}

	var tmpfile;
	if (typeof(doc.URL) != "undefined") {
		launchyShared.trace("URL <" + doc.URL + ">");
		if (launchyShared.isLinkType("file", doc.URL)) {
			tmpfile = LaunchyContext.fu.urlToPath(doc.URL);
		} else {
			var d = new Date().getTime();
			tmpfile = LaunchyContext.tmpdir.append("launchy-viewsource-" + d + ".html");
			LaunchyContext.tmpdir = LaunchyContext.tmpdir.parent;
			launchyShared.trace("Temporary file <" + tmpfile + ">");

			// var f=new File(tmpfile);
			// f.initPath(tmpfile);
			var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			f.initWithPath(tmpfile);
		
			var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);
			persist.persistFlags = 1 | 32 | 64 | 512 | 2048;
			launchyShared.trace("Trying to save URL <" + doc.URL + ">");
			try {
				var encode = 4;
				persist.saveDocument(doc, f, null, null, encode, 0);
			} catch (ex) {
				launchyShared.trace("Cannot save URL <" + doc.URL + "> due to:\n" + ex);
			}
		}

		var xargs = new Array();
		if (LaunchyContext.apps[app_entry].args)
			xargs = LaunchyContext.apps[app_entry].args.join(',').split(',');
	
		var u = false;
		for (var i=0; i < xargs.length; i++) {
			var tmp = launchyShared.replaceInsert(xargs[i], 1, tmpfile)
			if (tmp != xargs[i]) {
				xargs[i] = tmp;
				u = true;
			}
		}
		if (!u)
			xargs[xargs.length] = tmpfile;
	
		launchyShared.trace("About to run:\n" + app_exe + " " + xargs.join(" "));
		try {
			var stat = document.getElementById("statusbar-display");
			stat.label = "Running <" + app_exe + " " + xargs.join(" ") + ">";
		} catch(err) {
		}
		LaunchyContext.fu.run(app_exe, xargs);
	} else {
		launchyShared.trace("Unable to get URL from document");
	}
}

LaunchyContext.prototype.doSourceCleanup = function() {
	var contents=LaunchyContext.tmpdir.readDir();
	var re=/^launchy-viewsource-(\d+)\.html$/i;
	var u = new Date().getTime();
	for (var i = 0; i < contents.length; i++) {
		if (contents[i].isFile()) {
			var x = contents[i].leaf.match(re);
			if (x && x[1]) {
				// delete temp files older than one day
				if (u - parseInt(x[1]) > 60*60*24*1000) {
					launchyShared.trace("Deleting old view-source files <" + contents[i].path + ">");
					contents[i].remove();
				}
			}
		}
	}
}

// create the object
const launchyContext = new LaunchyContext();
launchyContext.doInit();
